-- Items we don't want to pick up:
local pick_disabled = {
	["coal"] = true,
	["stone"] = true,
	["iron-ore"] = true,
	["copper-ore"] = true,
}

local function select_entity_standard(event)
	global.piggyback = false
	local p = game.players[event.player_index]
	if p.selected then
		local item, locname = p.selected.name, p.selected.localised_name
		if item == "straight-rail" or item == "curved-rail" then item = "rail" end
		if not pick_disabled[item] then
			local ip = game.item_prototypes[item]
			if ip then  -- If the item has a prototype, then it's something we can pick up.
				local n = math.min(p.get_item_count(item), ip.stack_size)
				if n > 0 then
					-- Player has some of this item in their inventory.
					local held_stack = nil
					local already_holding = nil
					if p.cursor_stack.valid_for_read then
						-- Player is already holding something, remember what it was:
						held_stack = {name = p.cursor_stack.name, count = p.cursor_stack.count}
						if held_stack.name == item then
							already_holding = true
						else
							p.cursor_stack.clear()
						end
					end
					if not already_holding then
						-- Put items in the cursor stack:
						p.cursor_stack.set_stack({name = item, count = n})
						-- Remove an equal number from the main inventory:
						p.remove_item({name = item, count = n})
						-- Put whatever was originally being held back into player's inventory:
						if held_stack then
							p.insert(held_stack)
						end
					end
				else
					-- Player doesn't have any of this item:
					p.print({"msg-no-item-inv", locname})
				end
			end
		end
	end
end

local function select_entity_piggyback(event)
	global.piggyback = true
	local p = game.players[event.player_index]
	-- Only run if the cursor stack is empty:
	if not p.cursor_stack.valid_for_read then
		if p.selected then
			-- p.selected is the entity under the cursor (or nil if none).
			local item, locname = p.selected.name, p.selected.localised_name
			-- Rails: there's only one inventory item called "rail", but in the world there are two entities:
			if item == "straight-rail" or item == "curved-rail" then item = "rail" end
			if not pick_disabled[item] then
				local ip = game.item_prototypes[item]
				if ip then  -- If the item has a prototype, then it's something we can pick up.
					local n = math.min(p.get_item_count(item), ip.stack_size)
					if n > 0 then	
						-- Player has some of this item in their inventory.
						-- Remove up to a full stack from player's inventory...
						p.remove_item({name = item, count = n})
						-- ...and put them into the player's hand:
						p.cursor_stack.set_stack({name = item, count = n})
					else
						-- Player doesn't have any of this item:
						p.print({"msg-no-item-inv", locname})
					end
				end
			end
		end
	end
end

local function stack_changed(event)
	if global.piggyback then select_entity_piggyback(event) end
end

script.on_event("picker-select-piggyback", select_entity_piggyback)
script.on_event(defines.events.on_player_cursor_stack_changed, stack_changed)
script.on_event("picker-select-standard", select_entity_standard)